// test.cpp : Defines the entry point for the console application.
//

#include "jbig2en.h"

#define WIDTHBYTES(bits)    (((bits) + 31) / 32 * 4)
#define IS_WIN30_DIB(lpbi)  ((*(LPDWORD)(lpbi)) != 12)
#define DIB_HEADER_MARKER	((WORD) ('M' << 8) | 'B') 
WORD  DIBNumColors(LPSTR lpbi)
{
	WORD wBitCount;  // DIB bit count
	
			 /*  If this is a Windows-style DIB, the number of colors in the
			 *  color table can be less than the number of bits per pixel
			 *  allows for (i.e. lpbi->biClrUsed can be set to some value).
			 *  If this is the case, return the appropriate value.
	*/
	
	if (IS_WIN30_DIB(lpbi))
	{
		DWORD dwClrUsed;
		
		dwClrUsed = ((LPBITMAPINFOHEADER)lpbi)->biClrUsed;
		if (dwClrUsed != 0)
			return (WORD)dwClrUsed;
	}
	
	/*  Calculate the number of colors in the color table based on
	*  the number of bits per pixel for the DIB.
	*/
	if (IS_WIN30_DIB(lpbi))
		wBitCount = ((LPBITMAPINFOHEADER)lpbi)->biBitCount;
	else
		wBitCount = ((LPBITMAPCOREHEADER)lpbi)->bcBitCount;
	
	/* return number of colors based on bits per pixel */
	switch (wBitCount)
	{
	case 1:
		return 2;
		
	case 4:
		return 16;
		
	case 8:
		return 256;
		
	default:
		return 0;
	}
}

WORD  PaletteSize(LPSTR lpbi)
{
	/* calculate the size required by the palette */
	if (IS_WIN30_DIB (lpbi))
		return (WORD)(::DIBNumColors(lpbi) * sizeof(RGBQUAD));
	else
		return (WORD)(::DIBNumColors(lpbi) * sizeof(RGBTRIPLE));
}

LPSTR  FindDIBBits(LPSTR lpbi)
{
	return (lpbi + *(LPDWORD)lpbi + ::PaletteSize(lpbi));
}
DWORD  DIBWidth(LPSTR lpDIB)
{
	LPBITMAPINFOHEADER lpbmi;  // pointer to a Win 3.0-style DIB
	LPBITMAPCOREHEADER lpbmc;  // pointer to an other-style DIB
	
	/* point to the header (whether Win 3.0 and old) */
	
	lpbmi = (LPBITMAPINFOHEADER)lpDIB;
	lpbmc = (LPBITMAPCOREHEADER)lpDIB;
	
	/* return the DIB width if it is a Win 3.0 DIB */
	if (IS_WIN30_DIB(lpDIB))
		return lpbmi->biWidth;
	else  /* it is an other-style DIB, so return its width */
		return (DWORD)lpbmc->bcWidth;
}
DWORD  DIBHeight(LPSTR lpDIB)
{
	LPBITMAPINFOHEADER lpbmi;  // pointer to a Win 3.0-style DIB
	LPBITMAPCOREHEADER lpbmc;  // pointer to an other-style DIB
	
	/* point to the header (whether old or Win 3.0 */
	
	lpbmi = (LPBITMAPINFOHEADER)lpDIB;
	lpbmc = (LPBITMAPCOREHEADER)lpDIB;
	
	/* return the DIB height if it is a Win 3.0 DIB */
	if (IS_WIN30_DIB(lpDIB))
		return lpbmi->biHeight;
	else  /* it is an other-style DIB, so return its height */
		return (DWORD)lpbmc->bcHeight;
}

BOOL SaveDIB(HDIB hDib, LPCTSTR lpFileName) 
{ 
	BITMAPFILEHEADER    bmfHdr;     // Header for Bitmap file 
	LPBITMAPINFOHEADER  lpBI;       // Pointer to DIB info structure 
	HANDLE              fh;         // file handle for opened file 
	DWORD               dwDIBSize; 
	DWORD               dwWritten; 
	
	if (!hDib) 
		return FALSE; 
	
	fh = CreateFile(lpFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN, NULL); 
	
	if (fh == INVALID_HANDLE_VALUE) 
		return FALSE; 
	
	// Get a pointer to the DIB memory, the first of which contains 
	// a BITMAPINFO structure 
	
	lpBI = (LPBITMAPINFOHEADER)GlobalLock(hDib); 
	if (!lpBI) 
	{ 
		CloseHandle(fh); 
		return FALSE; 
	} 
	
	// Check to see if we're dealing with an OS/2 DIB.  If so, don't 
	// save it because our functions aren't written to deal with these 
	// DIBs. 
	
	if (lpBI->biSize != sizeof(BITMAPINFOHEADER)) 
	{ 
		GlobalUnlock(hDib); 
		CloseHandle(fh); 
		return FALSE; 
	} 
	
	// Fill in the fields of the file header 
	
	// Fill in file type (first 2 bytes must be "BM" for a bitmap) 
	
	bmfHdr.bfType = DIB_HEADER_MARKER;  // "BM" 
	
	// Calculating the size of the DIB is a bit tricky (if we want to 
	// do it right).  The easiest way to do this is to call GlobalSize() 
	// on our global handle, but since the size of our global memory may have 
	// been padded a few bytes, we may end up writing out a few too 
	// many bytes to the file (which may cause problems with some apps, 
	// like HC 3.0). 
	// 
	// So, instead let's calculate the size manually. 
	// 
	// To do this, find size of header plus size of color table.  Since the 
	// first DWORD in both BITMAPINFOHEADER and BITMAPCOREHEADER conains 
	// the size of the structure, let's use this. 
	
	// Partial Calculation 
	
	dwDIBSize = *(LPDWORD)lpBI + PaletteSize((char*)lpBI);   
	
	// Now calculate the size of the image 
	
	// It's an RLE bitmap, we can't calculate size, so trust the biSizeImage 
	// field 
	
	if ((lpBI->biCompression == BI_RLE8) || (lpBI->biCompression == BI_RLE4)) 
		dwDIBSize += lpBI->biSizeImage; 
	else 
	{ 
		DWORD dwBmBitsSize;  // Size of Bitmap Bits only 
		
		// It's not RLE, so size is Width (DWORD aligned) * Height 
		
		dwBmBitsSize = WIDTHBYTES((lpBI->biWidth)*((DWORD)lpBI->biBitCount)) * 
			lpBI->biHeight; 
		
		dwDIBSize += dwBmBitsSize; 
		
		// Now, since we have calculated the correct size, why don't we 
		// fill in the biSizeImage field (this will fix any .BMP files which  
		// have this field incorrect). 
		
		lpBI->biSizeImage = dwBmBitsSize; 
	} 
	
	
	// Calculate the file size by adding the DIB size to sizeof(BITMAPFILEHEADER) 
	
	bmfHdr.bfSize = dwDIBSize + sizeof(BITMAPFILEHEADER); 
	bmfHdr.bfReserved1 = 0; 
	bmfHdr.bfReserved2 = 0; 
	
	// Now, calculate the offset the actual bitmap bits will be in 
	// the file -- It's the Bitmap file header plus the DIB header, 
	// plus the size of the color table. 
	
	bmfHdr.bfOffBits = (DWORD)sizeof(BITMAPFILEHEADER) + lpBI->biSize + 
		PaletteSize((char*)lpBI); 
	
	// Write the file header 
	
	WriteFile(fh, (LPBYTE)&bmfHdr, sizeof(BITMAPFILEHEADER), &dwWritten, NULL); 
	
	// Write the DIB header and the bits -- use local version of 
	// MyWrite, so we can write more than 32767 bytes of data 
	
	WriteFile(fh, (LPBYTE)lpBI, dwDIBSize, &dwWritten, NULL); 
	
	GlobalUnlock(hDib); 
	CloseHandle(fh); 
	
	if (dwWritten == 0) 
		return FALSE; // oops, something happened in the write 
	else 
		return TRUE; // Success code 
} 

void GetModulePath(char *out_path,char *in_name)
{
	char *p;
	GetModuleFileName(NULL, out_path,256);
	p =strrchr(out_path,'\\');
	p[1]=0;
	strcat(out_path,in_name);
}

int main(int argc, char* argv[])
{
	char szInputTIFF[MAX_PATH];
	GetModuleFileName(NULL,szInputTIFF,MAX_PATH);
	CHAR drive[_MAX_DRIVE],dir[_MAX_DIR],fname[_MAX_FNAME],ext[_MAX_EXT];
	_splitpath(szInputTIFF, drive, dir, fname, ext );
	_makepath(szInputTIFF, drive, dir, "example", ".tif" );
	
	JBIG2SDK_SetLicenseKey("XXXX-XXXX-XXXX-XXXX");
	LPJBIG2DATA ptrJBIG2Data = JBIG2SDK_ConvertBWTIFF2JBIG2Data(szInputTIFF);
	
	HDIB hDIB = JBIG2SDK_ConvertJBIG2Data2DIB(ptrJBIG2Data);
	char szFilePath[_MAX_PATH];
	GetModulePath(szFilePath, "test.bmp");
	SaveDIB(hDIB, szFilePath);
	
	LPJBIG2DATA ptrJBIG2Data2 = JBIG2SDK_ConvertDIB2JBIG2Data(hDIB);
	HDIB hDIB2 = JBIG2SDK_ConvertJBIG2Data2DIB(ptrJBIG2Data2);
	GetModulePath(szFilePath, "test2.bmp");
	SaveDIB(hDIB2, szFilePath);

	if(ptrJBIG2Data)
	{
		JBIG2SDK_FreeJBIG2Data(ptrJBIG2Data);
		ptrJBIG2Data = NULL;
	}
	if(ptrJBIG2Data2)
	{
		JBIG2SDK_FreeJBIG2Data(ptrJBIG2Data2);
		ptrJBIG2Data2 = NULL;
	}
	if(hDIB)
	{
		JBIG2SDK_FreeHDIBData(hDIB);
		hDIB = NULL;
	}
	if(hDIB2)
	{
		JBIG2SDK_FreeHDIBData(hDIB2);
		hDIB2 = NULL;
	}
	return 0;
}
